﻿//////////////////////////////////////////////
// MeshManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Mesh ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT MeshManager final : public nkCommon::SingletonClass<MeshManager>
	{
		friend class nkCommon::SingletonClass<MeshManager> ;

		private :
		
			// Attributs
			std::unordered_map<std::string, Mesh*> _meshMap ;

			// Threading
			mutable std::mutex _mutex ;

			// Compteur d'id actif
			unsigned int _idCounter ;

		private :

			// Constructor
			MeshManager () ;
			
		public :

			// Destructor
			~MeshManager () ;

			// Gestion mémoire
			Mesh* createOrRetrieve (const nkMemory::StringView& name) ;
			Mesh* get (const nkMemory::StringView& name) const ;
			Mesh* getByIndex (unsigned int index) const ;
			void rename (const nkMemory::StringView& currentName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;
	} ;
}