﻿//////////////////////////////////////////////
// RenderContext.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Compositor ;
	class RenderContextDescriptor ;
	class Texture ;
	class Viewport ;
}

namespace nkWinUi
{
	class Window ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderContext
	{
		protected :
		
			// Attributes
			// Size information
			int _width ;
			int _height ;
			// Back buffer used
			Texture* _backBuffer ;
			// Depth target for ease of use
			Texture* _depthBuffer ;
			// Viewport attached
			Viewport* _viewport ;

			// If a compositor is linked
			Compositor* _compositor ;

			// Attached window
			nkWinUi::Window* _attachedWin ;
			bool _ownsSurface ;
			bool _vsync ;
		
		public :
		
			// Constructor, destructor
			RenderContext () ;
			virtual ~RenderContext () ;

			// Getters
			Texture* getBackBuffer () const ;
			Texture* getDepthBuffer () const ;
			Viewport* getViewport () const ;
			Compositor* getCompositor () const ;
			int getWidth () const ;
			int getHeight () const ;
			bool isWindowFocused () const ;
			nkWinUi::Window* getAttachedWin () const ;
			bool getOwnsSurface () const ;

			// Setters
			void setCompositor (Compositor* value) ;
			void setOwnsSurface (bool value) ;
			virtual void setGammaCorrected (bool value) = 0 ;

			// Initialization
			virtual void initialize (const RenderContextDescriptor& descriptor) = 0 ;

			// Rendering
			virtual void swap () = 0 ;

			// Chain manipulation
			virtual void refreshBufferSize (unsigned int width = 0, unsigned int height = 0) = 0 ;
			virtual void refreshViewportSize () ;

			// TODO if perfs are worth it
			//void enableUavManipulation (bool value) ;
	} ;
}